#! perl -w

# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 1991, 2005 All Rights Reserved. 


package rterror;

# Tags look like this

$startpattern   = '// {{{RME (.*)$';
$endpattern     = '// }}}RME$';
$ignorepattern1 = '// {{{USR$';
$ignorepattern2 = '// }}}USR$';

%qualifies =
	(
		'associationEnd' => 'classifier',
		'capsuleRole'    => 'classifier',
		'choicePoint'    => 'classifier',
		'classAttribute' => 'classifier',
		'classifier'     => '.',
		'component'      => '.',
		'connector'      => 'classifier',
		'enter'          => 'classifier',
		'exit'           => 'classifier',
		'general'        => '*',
		'guard'          => 'transition',
		'inSignal'       => 'classifier',
		'operation'      => 'classifier',
		'outSignal'      => 'classifier',
		'package'        => '.',
		'parameter'      => '*',
		'port'           => 'classifier',
		'property'       => '*',
		'state'          => 'classifier',
		'tool'           => '*',
		'transition'     => 'classifier',
		'uniqueId'       => 'uniqueId'
	);

# print_error - Subroutine to print out an error
# Arguments
#   <object>    object of error
#   <message>   string containing the message
#   <level>     level giving the severity
#       0 = error
#       1 = warning

$maxError      = 2;
$worstError    = $maxError;
$componentName = '';

@levelName = ( 'error', 'warning' );

sub print_error($$$)
{
	my( $object, $msg, $level ) = @_;

	print 'GES';
	print ' ', $object if $object;

	$msg =~ s/'/''/g;

	print " description '", $msg,
		  "' severity '",   $levelName[ $level ],
		  "'\n";

	$worstError = $level if $worstError > $level;
}

sub handle_errors()
{
	my $error;

	error_loop: foreach $error ( @main::errors )
	{
		chomp $error;

		foreach $handler ( @config::handlers )
		{
			( $pattern, $action ) = @$handler;

			if( $error =~ /$pattern/ )
			{
				&$action;
				next error_loop;
			}
		}

		# not handled
		# &print_error( '', $error, 1 );
	}
}

# Handlers

sub action_ignore
{
}

sub action_print($$$$)
{
	my( $file, $lineno, $msg, $level ) = @_;

	if( open FILE, $file )
	{
		# found file directly
	}
	elsif( $main::srcdir && open FILE, "$main::srcdir/$file" )
	{
		# found file in source directory
	}
	else
	{
		&print_error( '', "$file:$lineno - $msg", $level );
		return;
	}

	my $object      = '';
	my $gesmsg      = '';
	my @taglineno   = ();
	my @bracelineno = ();
	my @taginfo     = ();
	my $filelineno  = 0;
	my $fileline;

	while( $fileline = <FILE> )
	{
		++$filelineno;
		chomp $fileline;

		if( $filelineno == $lineno )
		{
			if( $#taglineno >= 0 )
			{
				my $baselineno = $bracelineno[$#bracelineno] ||
								 $taglineno  [$#taglineno  ];

				$lineno -= $baselineno;

				my %tags    = ();
				my $want    = '*';
				my $tagtext = '';

				while( $tagtext = pop @taginfo )
				{
					while( $tagtext =~ s/\s*(\w+)\s+'([^']+)'\s*$// )
					{
						my( $key, $value ) = ( $1, $2 );

						# only consider this key if we need it
						next if $want ne '*' && $key ne $want;

						my $next = $qualifies{$key};

						# skip unrecognized keys
						next unless $next;

						$want = $next;

						if( $key eq 'classifier' )
						{
							if( $value !~ m{::} )
							{
								# not fully qualified: keep looking
								$want = $key;
							}

							$value .= '::' . $tags{$key} if $tags{$key};
						}

						$tags{$key} = $value;
					}

					last if $want eq '.';
				}

				$tags{'line'} = $lineno;

				my @list = ();

				foreach $key ( sort keys %tags )
				{
					push @list, $key, "'$tags{$key}'";
				}

				$object = join( ' ', @list );
				$gesmsg = $msg;
			}
			last;
		}
		elsif( $fileline =~ m{$startpattern}o )
		{
			push @taginfo,     $1;
			push @taglineno,   $filelineno;
			push @bracelineno, 0;
		}
		elsif( $fileline =~ m{$endpattern}o )
		{
			pop @bracelineno;
			pop @taglineno;
			pop @taginfo;
		}
		elsif( $fileline =~ m{$ignorepattern1}o
			or $fileline =~ m{$ignorepattern2}o )
		{
			++$taglineno  [$#taglineno  ] if $#taglineno   >= 0;
			++$bracelineno[$#bracelineno] if $#bracelineno >= 0;
		}
		elsif( $fileline =~ m{^\s*\{$} ) # }
		{
			# We are only interested in the first
			# opening brace to occur after a start tag.
			if( $#bracelineno >= 0 && $bracelineno[$#bracelineno] == 0 )
			{
				$bracelineno[$#bracelineno] = $filelineno;
			}
		}
	}

	close FILE;

	$gesmsg = "$file:$lineno - $msg" unless $gesmsg;

	&print_error( $object, $gesmsg, $level );
}

sub action_message($$)
{
	my( $msg, $level ) = @_;

	if( $componentName ne '' )
	{
		&print_error( "component '$componentName'", $msg, $level );
	}
	else
	{
		&print_error( '', $msg, $level );
	}
}

return 1;
